import java.net.*;  
import java.io.*; 
import java.util.*; 
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousServerSocketChannel;
import java.nio.channels.AsynchronousSocketChannel;
import java.net.InetSocketAddress;
import java.nio.channels.CompletionHandler;
import java.nio.channels.*;

public class NetworkClient 
{        
    private ArrayList<HeroStats> list = new ArrayList<HeroStats>(); 
    private ArrayList<Shot> myShotList = new ArrayList<Shot>(); 
    private ArrayList<Shot> shotList = new ArrayList<Shot>(); 
    private HeroStats refreshStat;

    protected int id = -1;
    protected boolean connected = false;

    //String address = "160.217.222.200";
    //String address = "192.168.1.150";
    //String address = "127.0.0.1";
    String address = "88.100.127.215";
    private InetAddress host;
    private int port = 16621;
    private int serverPort = 16620;

    private Socket s = null;
    private Socket s2 = null;
    private DatagramSocket dsoc = null;

    OutputStream os = null;
    ObjectOutputStream oos = null;

    InputStream is = null;
    ObjectInputStream ois = null;
    //DataInputStream din = null;

    public NetworkClient()
    {
        establishConnection();
        Random random = new Random();
        id = random.nextInt();
    }

    public ArrayList<Shot> getShotList()
    {
        return shotList;
    }

    public ArrayList<HeroStats> getCollection()
    {
        return list;
    }

    public void makeShot(HeroStats stat)
    {
        boolean isLeft = stat.isLeft, isStanding = stat.isStanding;
        if(isStanding)
            if(isLeft)
                myShotList.add(new Shot(stat.isLeft, (int)stat.x, (int)stat.y+10, id));
            else
                myShotList.add(new Shot(stat.isLeft, (int)stat.x+30, (int)stat.y+10, id));
        else
            if(isLeft)
                myShotList.add(new Shot(stat.isLeft, (int)stat.x, (int)stat.y+30, id));
            else
                myShotList.add(new Shot(stat.isLeft, (int)stat.x+30, (int)stat.y+30, id));
    }

    public int getHealth()
    {
        for(HeroStats s : list)
        {
            if(id == s.id)
                return s.health;
        }
        //System.out.println("HP not found");
        return 100;
    }

    public void establishConnection()
    {
        try
        {
            dsoc = new DatagramSocket(port); 
            host = InetAddress.getByName(address);
            
            receive();

            connected = true;
        }
        catch(IOException e)
        {
            e.printStackTrace();
            System.out.println("Cannot establish connection");
            connected = false;
        }
    }

    public void send(HeroStats stats) throws IOException
    {
        try
        {
            stats.id = id;
            byte[] buffer = gameObjectsToBytes(stats, myShotList);
            dsoc.send(new DatagramPacket(buffer, buffer.length, host, serverPort));
            myShotList.clear();
        }
        catch(IOException e)
        {
            connected = false;
            e.printStackTrace();
            System.out.println("Server IO send error");
        }
        catch(Exception e)
        {
            connected = false;
            e.printStackTrace();
            System.out.println("Server send error");
        }
    }

    public void receive()
    {
        Thread refresh = new Thread()
            {
                public void run()
                {
                    boolean stop = false;
                    while(!stop)
                    {
                        byte buffer[] = new byte[4096];
                        DatagramPacket dpack = new DatagramPacket(buffer,buffer.length);
                        ObjectInput in = null;
                        ByteArrayInputStream bis = null;
                        try
                        {
                            //System.out.println("PRJIMAM NECO?");
                            dsoc.receive(dpack);
                            //System.out.println("ano");
                            bis = new ByteArrayInputStream(buffer);
                            in = new ObjectInputStream(bis);
                            list = (ArrayList<HeroStats>)(in.readObject());
                            shotList = (ArrayList<Shot>)(in.readObject()); 
                        }
                        catch(IOException e)
                        {
                            e.printStackTrace();
                            System.out.println("Receive packet IO error");
                        }
                        catch(Exception e)
                        {
                            e.printStackTrace();
                            System.out.println("Receive packet error");
                        }
                        finally 
                        {
                            try
                            {
                                bis.close();
                                if (in != null)
                                    in.close();
                            }
                            catch(Exception e) {}
                        } 

                    }
                }
            };
        refresh.start();
    }

    public byte[] gameObjectsToBytes(HeroStats stat, ArrayList<Shot> shots) throws Exception
    {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectOutput out = null;
        byte[] bytes = new byte[4096];
        try 
        {
            out = new ObjectOutputStream(bos);   
            out.writeObject(stat);
            out.writeObject(shots);
            bytes = bos.toByteArray();
        } 
        finally 
        {
            try {
                if (out != null) {
                    out.close();
                }
            } catch (IOException ex) {
                // ignore close exception
            }
            try {
                bos.close();
            } catch (IOException ex) {
                // ignore close exception
            }
        }
        return bytes;
    } 
}